/**
 * Property decorator that defines a cloning depth for @Immutable.
 *
 * The default cloning depth of 0 (@CloneDepth(0) can be omitted) means that every property
 * of the class decorated as @Immutable will be copied by-value ("flat cloned") when a property
 * of an instance is called. A depth of 1 copies every property of the property on method call.
 *
 * @example
 *     @Immutable()
 *     class Example {
 *         @CloneDepth(0)
 *         copiedByReferenceA = { prop: 1 };
 *
 *         @CloneDepth(1)
 *         copiedByValueB = { copiedByReferenceB: { prop: 2 } };
 *
 *         @CloneDepth(2)
 *         copiedByValueC = { alsoCopiedByValueC: { copiedByReferenceC: { prop: 3 } } };
 *     }
 */
export declare function CloneDepth(depth: number): <T extends {
    constructor: Function;
}>(prototype: T, key: string, dontUseOnMethods?: undefined) => void;
