export declare type DeepPartial<T> = {
    [P in keyof T]?: T[P] | DeepPartial<T[P]>;
};
/**
 * Applies all changes deeply on an object.
 * If any object would have the same values as before, the same reference is used.
 * If any properties changed, returns a new Object that contains all values of a overwritten with values of b
 * Arrays are overwritten (with reference reuse), _not_ merged.
 *
 * @example
 * const newPoint = { pos: {x: 1, y: 2}, color: 'red' };
 * const oldPoint = { pos: {x: 1, y: 2}, color: 'green' };
 * const result = deepApplyWithReuse(oldPoint, newPoint);
 * // => result.pos === oldPoint.pos
 * // => result.color === 'red'
 */
export declare function deepApplyWithReuse<T extends Object>(a: T, b: DeepPartial<T>): T;
