import { Subscribable, TrackedMethodCall } from './immutable-interfaces';
import { StateActionBranch } from './state-action-branch';
/**
 * Implements an observable store with separate state action branches.
 */
export declare class ImmutableStateStore<StateType, ActionsType extends {
    [key: string]: StateActionBranch<StateType>;
}> {
    get state(): StateType;
    readonly actions: ActionsType;
    private currentState;
    private observers;
    private callObservers;
    private subscriptions;
    constructor(actions: ActionsType);
    /** Remove all observers and free resources. */
    destroy(): void;
    /** Observe calls of state action methods as an Observable of the passed Implementation. */
    observeCalls<ObservableType extends Subscribable<TrackedMethodCall<StateType>>>(observableClass: {
        new <T>(): ObservableType;
    }): ObservableType;
    /** Observe calls of state action methods as plain-JS observable. */
    observeCalls(): Subscribable<TrackedMethodCall<StateType>>;
    /** Observe the application state as an Observable of the passed Implementation. */
    observeState<ObservableType extends Subscribable<StateType>>(observableClass: {
        new <T>(): ObservableType;
    }): ObservableType;
    /** Observe the application state as a plain-JS observable. */
    observeState(): Subscribable<StateType>;
    /** Fully replace the application state (e.g. for undo/redo) */
    replaceState(newState: StateType): void;
    /** Should only be used in testing. Use action methods to change the application state. */
    setStateForTesting(newState: Partial<StateType>): void;
    private combineInitialState;
    private observeActionMethodCalls;
    private propertiesChanged;
    private emitChangedState;
    private emitMethodCall;
    private handleSubscription;
}
