"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CloneDepth = void 0;
var utils_1 = require("./utils");
/**
 * Property decorator that defines a cloning depth for @Immutable.
 *
 * The default cloning depth of 0 (@CloneDepth(0) can be omitted) means that every property
 * of the class decorated as @Immutable will be copied by-value ("flat cloned") when a property
 * of an instance is called. A depth of 1 copies every property of the property on method call.
 *
 * @example
 *     @Immutable()
 *     class Example {
 *         @CloneDepth(0)
 *         copiedByReferenceA = { prop: 1 };
 *
 *         @CloneDepth(1)
 *         copiedByValueB = { copiedByReferenceB: { prop: 2 } };
 *
 *         @CloneDepth(2)
 *         copiedByValueC = { alsoCopiedByValueC: { copiedByReferenceC: { prop: 3 } } };
 *     }
 */
function CloneDepth(depth) {
    return function CloneDepthDecorator(prototype, key) {
        var _a;
        var existingMetadata = utils_1.getImmutableClassMetadata(prototype.constructor);
        var cloneDepth = existingMetadata && existingMetadata.cloneDepth || {};
        var metaData = __assign(__assign({}, (existingMetadata || {})), { cloneDepth: __assign(__assign({}, cloneDepth), (_a = {}, _a[key] = depth, _a)) });
        utils_1.setImmutableClassMetadata(prototype.constructor, metaData);
    };
}
exports.CloneDepth = CloneDepth;

//# sourceMappingURL=clone-depth-decorator.js.map
