"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deepClone = void 0;
var utils_1 = require("./utils");
function deepClone(original, depth) {
    if (depth === void 0) { depth = Number.POSITIVE_INFINITY; }
    if (!original || depth < 0) {
        return original;
    }
    else if (utils_1.isArray(original)) {
        return original.map(function (val) { return deepClone(val, depth - 1); });
    }
    var typeofOriginal = typeof original;
    if (typeofOriginal === 'object') {
        var prototype = utils_1.objectGetPrototypeOf(original);
        if (prototype === Object.prototype) {
            return depth ? clonePropsTo({}, original, depth) : utils_1.objectAssign({}, original);
        }
        else if (prototype === RegExp.prototype) {
            return new RegExp(original);
        }
        else if (prototype === Date.prototype) {
            return new Date(original);
        }
        else if (typeof Map === 'function' && prototype === Map.prototype) {
            return new Map(original);
        }
        else if (typeof Set === 'function' && prototype === Set.prototype) {
            return new Set(original);
        }
        else if (typeof Node === 'function' && original instanceof Node && typeof original.cloneNode === 'function') {
            return original.cloneNode(true);
        }
        var clone = utils_1.objectCreate(prototype);
        return clonePropsTo(clone, original, depth);
    }
    else if (typeofOriginal === 'function') {
        var clone = function () {
            return original.apply(this, arguments);
        };
        return clonePropsTo(clone, original, depth);
    }
    else {
        return original;
    }
}
exports.deepClone = deepClone;
function clonePropsTo(target, source, depth) {
    for (var _i = 0, _a = utils_1.objectKeys(source); _i < _a.length; _i++) {
        var key = _a[_i];
        target[key] = deepClone(source[key], depth - 1);
    }
    return target;
}

//# sourceMappingURL=deep-clone.js.map
