"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deepEqual = void 0;
var utils_1 = require("./utils");
function deepEqual(a, b) {
    if (a === b) {
        return true;
    }
    if (typeof a !== typeof b) {
        return false;
    }
    if (a == null || b == null) {
        return false;
    }
    if (typeof a === 'object') {
        var arrayA = utils_1.isArray(a);
        var arrayB = utils_1.isArray(b);
        if (arrayA !== arrayB) {
            return false;
        }
        else if (arrayA) {
            // Array
            if (a.length !== b.length) {
                return false;
            }
            var len = a.length;
            for (var i = 0; i < len; i++) {
                if (!deepEqual(a[i], b[i])) {
                    return false;
                }
            }
            return true;
        }
        else {
            // Object
            var protoA = utils_1.objectGetPrototypeOf(a);
            var protoB = utils_1.objectGetPrototypeOf(b);
            if (protoA !== protoB) {
                return false;
            }
            else if (protoA === Date.prototype) {
                return Number(a) === Number(b);
            }
            else if (protoA === RegExp.prototype) {
                return a.toString() === b.toString();
            }
            // Plain object or custom prototype
            var keysA = utils_1.objectKeys(a).sort();
            var keysB = utils_1.objectKeys(b).sort();
            if (keysA.length !== keysB.length) {
                return false;
            }
            for (var i = 0; i < keysA.length; i++) {
                var key = keysA[i];
                if (key !== keysB[i]) {
                    return false;
                }
                if (!deepEqual(a[key], b[key])) {
                    return false;
                }
            }
            return true;
        }
    }
    else if (typeof a === 'number') {
        // Let NaN === NaN be true, unlike in normal JS
        return Number.isNaN(a) && Number.isNaN(b);
    }
    else {
        // Non-equal primitive type or function
        return false;
    }
}
exports.deepEqual = deepEqual;

//# sourceMappingURL=deep-equal.js.map
