"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findMutatedProperties = void 0;
var mutateCache = {};
var sourceRx = /^\s*(?:function [^\(]*?\(([^,)]+?)\) ?\{|([^\s,]+?) ?=>\s*)\{?\s*([\s\S]+?)\s*\}?$/;
/**
 * Try to parse the body of the passed function to find out which properties will be changed.
 * @internal
 */
function findMutatedProperties(fn) {
    var functionSource = fn.toString();
    if (mutateCache[functionSource]) {
        return mutateCache[functionSource];
    }
    var match = functionSource.match(sourceRx);
    if (!match) {
        return {
            simple: false
        };
    }
    var paramName = match[2] || match[1] || '';
    var body = match[3];
    var regex = new RegExp("(?:(?:^|\\W)" + paramName + "(?:(\\[)|\\.([\\w]+)\\s*(?:[-+/*%]?=|\\+\\+|--))|(?:--|\\+\\+)" + paramName + "\\.([\\w]+)|(" + paramName + "))", 'g');
    var found;
    var props = [];
    while (found = regex.exec(body)) {
        // obj[] or fn(obj) was found - we can't parse that.
        if (found[1] || found[4]) {
            return mutateCache[functionSource] = { simple: false };
        }
        // obj.propname was found - add to list.
        var propName = found[2] || found[3];
        if (props.indexOf(propName) < 0) {
            props.push(propName);
        }
    }
    return mutateCache[functionSource] = { simple: true, props: props };
}
exports.findMutatedProperties = findMutatedProperties;

//# sourceMappingURL=function-changes.js.map
