"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.functionMutatesInput = void 0;
var traverse_object_1 = require("./traverse-object");
var utils_1 = require("./utils");
function functionMutatesInput(fn, args, thisArg) {
    if (thisArg === void 0) { thisArg = undefined; }
    var thisSnapshot = createSnapshot(thisArg);
    var argSnapshots = args.map(createSnapshot);
    fn.apply(thisArg, args);
    var thisChanges = checkSnapshotChanges(thisSnapshot);
    var argChanges = {};
    argSnapshots.forEach(function (snapshot, index) {
        var changes = checkSnapshotChanges(snapshot);
        if (changes.length > 0) {
            argChanges[index] = changes;
        }
    });
    if (thisChanges.length > 0 || utils_1.objectKeys(argChanges).length > 0) {
        return {
            args: argChanges,
            this: thisChanges
        };
    }
    else {
        return false;
    }
}
exports.functionMutatesInput = functionMutatesInput;
function createSnapshot(object) {
    var map = new Map();
    traverse_object_1.traverseObject(object, function (prop, path) {
        map.set(prop, { value: utils_1.objectAssign({}, prop), path: path });
    });
    return map;
}
function checkSnapshotChanges(snapshotMap) {
    var changes = [];
    snapshotMap.forEach(function (snapshot, object) {
        var newKeys = utils_1.objectKeys(object);
        var oldKeys = utils_1.objectKeys(snapshot.value);
        for (var _i = 0, _a = oldKeys.filter(function (k) { return newKeys.indexOf(k) < 0; }); _i < _a.length; _i++) {
            var key = _a[_i];
            changes.push({
                path: __spreadArrays(snapshot.path, [key]),
                oldValue: snapshot.value[key]
            });
        }
        for (var _b = 0, newKeys_1 = newKeys; _b < newKeys_1.length; _b++) {
            var key = newKeys_1[_b];
            var newValue = object[key];
            var oldValue = snapshot.value[key];
            if (oldKeys.indexOf(key) < 0) {
                changes.push({
                    path: __spreadArrays(snapshot.path, [key]),
                    newValue: newValue
                });
            }
            else if (newValue !== oldValue && !utils_1.bothEqualNaN(newValue, oldValue)) {
                changes.push({
                    path: __spreadArrays(snapshot.path, [key]),
                    oldValue: oldValue,
                    newValue: newValue
                });
            }
        }
    });
    return changes;
}

//# sourceMappingURL=function-mutates-input.js.map
