"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isConstructingImmutable = exports.restoreUnchangedProperties = exports.createImmutableClass = exports.Immutable = void 0;
var deep_clone_1 = require("./deep-clone");
var flat_equal_1 = require("./flat-equal");
var function_mutates_input_1 = require("./function-mutates-input");
var immutable_settings_1 = require("./immutable-settings");
var method_not_immutable_error_1 = require("./method-not-immutable-error");
var deep_freeze_1 = require("./deep-freeze");
var utils_1 = require("./utils");
var constructing = false;
function Immutable(originalClass) {
    if (originalClass) {
        return createImmutableClass(originalClass);
    }
    else {
        return Immutable;
    }
}
exports.Immutable = Immutable;
/** @internal */
function createImmutableClass(originalClass) {
    var originalMetadata = utils_1.getImmutableClassMetadata(originalClass);
    var metadata = {
        cloneDepth: originalMetadata && originalMetadata.cloneDepth || {},
        originalClass: originalClass,
        settings: immutable_settings_1.globalSettings,
    };
    // Constructor that will be called on `new Class()` instead of the original constructor.
    function mappedConstructor() {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        constructing = true;
        var instance = new (originalClass.bind.apply(originalClass, __spreadArrays([void 0], args)))();
        constructing = false;
        for (var _a = 0, _b = Object.getOwnPropertyNames(instance); _a < _b.length; _a++) {
            var key = _b[_a];
            var descriptor = Object.getOwnPropertyDescriptor(instance, key);
            utils_1.objectDefineProperty(this, key, descriptor);
        }
        8;
        utils_1.setInitialInstanceMetadata(this);
        if (metadata.settings.deepFreeze) {
            for (var _c = 0, _d = utils_1.objectKeys(this); _c < _d.length; _c++) {
                var key = _d[_c];
                var propertyValue = this[key];
                if (typeof propertyValue === 'object' && propertyValue) {
                    deep_freeze_1.deepFreeze(propertyValue);
                }
            }
        }
    }
    var className = utils_1.getFunctionName(originalClass);
    var mappedClass = createNamedFunction(className, mappedConstructor);
    var originalPrototype = originalClass.prototype;
    var mappedPrototype = mappedClass.prototype = utils_1.objectCreate(originalClass.prototype);
    // Map methods of the original class
    for (var _i = 0, _a = Object.getOwnPropertyNames(originalPrototype); _i < _a.length; _i++) {
        var propertyKey = _a[_i];
        var method = originalPrototype[propertyKey];
        if (propertyKey !== 'constructor' && typeof method === 'function') {
            mappedPrototype[propertyKey] = createMethodWrapper(method, propertyKey, metadata);
        }
    }
    utils_1.objectDefineProperty(mappedPrototype, 'constructor', {
        configurable: true,
        enumerable: false,
        value: mappedClass,
        writable: true
    });
    // Copy static methods and properties
    for (var _b = 0, _c = Object.getOwnPropertyNames(originalClass); _b < _c.length; _b++) {
        var staticKey = _c[_b];
        if (!utils_1.hasOwnProperty(mappedClass, staticKey)) {
            var descriptor = Object.getOwnPropertyDescriptor(originalClass, staticKey);
            utils_1.objectDefineProperty(mappedClass, staticKey, descriptor);
        }
    }
    // Define immutable-specific metadata
    utils_1.setImmutableClassMetadata(mappedClass, metadata);
    utils_1.setImmutableClassMetadata(mappedPrototype, metadata);
    return mappedClass;
}
exports.createImmutableClass = createImmutableClass;
/** Creates a new function with the passed name. */
function createNamedFunction(name, realImplementation) {
    name = name.replace(/[+-.,;()\{\}\[\]]+/g, '_');
    var creator = new Function('fn', "return function " + name + "() { return fn.apply(this, arguments); }");
    return creator(realImplementation);
}
function createMethodWrapper(originalMethod, methodName, classMetadata) {
    var wrapped = function immutableWrappedMethod() {
        var _this = this;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var cloneDepth = classMetadata.cloneDepth, originalClass = classMetadata.originalClass, settings = classMetadata.settings;
        var instanceMetadata = (utils_1.getInstanceMetadata(this) || {});
        var originalProperties = {};
        var returnValue;
        var timeBefore;
        var callDuration = -1;
        // Only check mutability when enabled (e.g. in development)
        var runAndCheckChanges = settings.checkMutability
            ? function_mutates_input_1.functionMutatesInput
            : (function (callback) { callback(); return false; });
        var mutations = runAndCheckChanges(function () {
            for (var _i = 0, _a = utils_1.objectKeys(_this); _i < _a.length; _i++) {
                var key = _a[_i];
                originalProperties[key] = _this[key];
                var propertyCloneDepth = cloneDepth[key];
                if (propertyCloneDepth > 0) {
                    _this[key] = deep_clone_1.deepClone(_this[key], propertyCloneDepth - 1);
                }
            }
            timeBefore = new Date();
            var timestampBefore = getTimestamp();
            instanceMetadata.callDepth++;
            try {
                // Finally call the actual method
                returnValue = originalMethod.apply(_this, args);
                callDuration = getTimestamp() - timestampBefore;
            }
            finally {
                instanceMetadata.callDepth--;
            }
            // Restore unchanged properties
            restoreUnchangedProperties(_this, originalProperties, cloneDepth);
        }, args, this);
        if (mutations) {
            // Throw when a method changes "this.prop.otherProp", but not "this.prop"
            if (mutations.this.some(function (change) { return change.path.length > 1; }) || Object.keys(mutations.args).length > 0) {
                // A method has changed a property without cloning the object first
                throw new method_not_immutable_error_1.MethodNotImmutableError(mutations, originalMethod, originalClass);
            }
        }
        if (classMetadata.settings.deepFreeze) {
            for (var _a = 0, _b = utils_1.objectKeys(this); _a < _b.length; _a++) {
                var key = _b[_a];
                var propertyValue = this[key];
                if (propertyValue !== originalProperties[key] && typeof propertyValue === 'object' && propertyValue) {
                    deep_freeze_1.deepFreeze(propertyValue);
                }
            }
        }
        // Notify all observers added by observeImmutable
        var observers = instanceMetadata.observers;
        if (observers && observers.length > 0 && !instanceMetadata.callDepth) {
            var call = {
                instance: this,
                methodName: methodName,
                arguments: args,
                returnValue: returnValue,
                callDuration: callDuration,
                changes: undefined,
                oldProperties: originalProperties,
                newProperties: utils_1.objectAssign({}, this)
            };
            var hasChanges = false;
            var changes = {};
            for (var _c = 0, _d = utils_1.objectKeys(this); _c < _d.length; _c++) {
                var key = _d[_c];
                if (this[key] !== originalProperties[key]) {
                    hasChanges = true;
                    changes[key] = {
                        oldValue: originalProperties[key],
                        newValue: this[key]
                    };
                }
            }
            if (hasChanges) {
                call.changes = changes;
            }
            for (var _e = 0, observers_1 = observers; _e < observers_1.length; _e++) {
                var observer = observers_1[_e];
                observer(call);
            }
        }
        return returnValue;
    };
    return wrapped;
}
/** @internal */
function restoreUnchangedProperties(target, original, depth) {
    for (var _i = 0, _a = utils_1.objectKeys(target); _i < _a.length; _i++) {
        var key = _a[_i];
        if (typeof target[key] === 'object' && target[key] !== null) {
            var propDepth = typeof depth === 'number' ? depth : (depth[key] || 0);
            if (propDepth > 0) {
                restoreUnchangedProperties(target[key], original[key], propDepth - 1);
                if (target[key] !== original[key] && flat_equal_1.flatEqual(target[key], original[key])) {
                    target[key] = original[key];
                }
            }
        }
    }
}
exports.restoreUnchangedProperties = restoreUnchangedProperties;
/** @internal */
function isConstructingImmutable() {
    return constructing;
}
exports.isConstructingImmutable = isConstructingImmutable;
/** @internal */
function getTimestamp() {
    if (typeof performance === 'object') {
        // Cut values like 1234.8850000001 to 1234.885
        return (performance.now() * 1000 | 0) * 0.001;
    }
    else if (Date.now) {
        return Date.now();
    }
    else {
        return +(new Date());
    }
}

//# sourceMappingURL=immutable-decorator.js.map
