"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isCyclicStructure = void 0;
var utils_1 = require("./utils");
/**
 * Returns true if the passed object contains any object reference that contains itself as a child property.
 * If the function returns false, the passed object can be traversed recursively without causing an infinite loop.
 */
function isCyclicStructure(object) {
    if (typeof object !== 'object' || object == null) {
        return false;
    }
    return isCyclic(object, [], []);
}
exports.isCyclicStructure = isCyclicStructure;
/** @internal */
function isCyclic(object, parents, traversed) {
    if (parents.indexOf(object) >= 0) {
        return true;
    }
    if (traversed.indexOf(object) >= 0) {
        return false;
    }
    traversed.push(object);
    parents = __spreadArrays(parents, [object]);
    if (utils_1.isArray(object)) {
        for (var _i = 0, object_1 = object; _i < object_1.length; _i++) {
            var value = object_1[_i];
            if (typeof value === 'object' && value != null && isCyclic(value, parents, traversed)) {
                return true;
            }
        }
    }
    else {
        for (var _a = 0, _b = utils_1.objectKeys(object); _a < _b.length; _a++) {
            var key = _b[_a];
            if (typeof object[key] === 'object' && object[key] != null && isCyclic(object[key], parents, traversed)) {
                return true;
            }
        }
    }
    return false;
}

//# sourceMappingURL=is-cyclic-structure.js.map
