"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.pathToString = exports.MethodNotImmutableError = void 0;
var utils_1 = require("./utils");
var paddingWhitespace = '                                ';
/**
 * Error thrown when a method is marked as immutable but modifies its input values.
 */
var MethodNotImmutableError = /** @class */ (function (_super) {
    __extends(MethodNotImmutableError, _super);
    function MethodNotImmutableError(mutations, originalMethod, originalConstructor, methodName) {
        var _this = this;
        if (!methodName) {
            var prototype = originalConstructor && originalConstructor.prototype || undefined;
            methodName = utils_1.getFunctionName(originalMethod, prototype);
        }
        var className = originalConstructor && utils_1.getFunctionName(originalConstructor);
        var message = (className ? className + '.' : '') + methodName + ' mutates properties.';
        _this = _super.call(this, message) || this;
        // Fix for TypeScript when emitting to { target: "es5" }
        if (!(_this instanceof MethodNotImmutableError)) {
            Object.setPrototypeOf(_this, MethodNotImmutableError.prototype);
        }
        utils_1.objectDefineProperty(_this, 'name', {
            configurable: true,
            value: 'MethodNotImmutableError',
            writable: true
        });
        var argNames = utils_1.getFunctionArgs(originalMethod);
        utils_1.objectDefineProperty(_this, 'diff', {
            configurable: true,
            value: createMutationDiff(mutations, argNames),
            writable: false
        });
        _this.class = className;
        _this.message = message;
        _this.method = methodName;
        return _this;
    }
    MethodNotImmutableError.prototype.toString = function () {
        return this.name + ': ' + this.message + '\n\nChanges:\n' + this.diff;
    };
    return MethodNotImmutableError;
}(Error));
exports.MethodNotImmutableError = MethodNotImmutableError;
function createMutationDiff(mutations, argNames) {
    var _a;
    var allChanges = (_a = mutations.this
        .map(function (prop) { return ({
        prop: prop,
        path: 'this' + pathToString(prop.path)
    }); }))
        .concat.apply(_a, utils_1.objectKeys(mutations.args)
        .map(function (index) { return mutations.args[+index]
        .map(function (prop) { return ({
        prop: prop,
        path: (argNames[+index] || '<argument ' + index + '>') + pathToString(prop.path)
    }); }); }));
    var added = allChanges
        .filter(function (change) { return 'newValue' in change.prop; })
        .map(function (change) { return ({
        sign: '+',
        path: change.path,
        value: formatJsValue(change.prop.newValue)
    }); });
    var removed = allChanges
        .filter(function (change) { return 'oldValue' in change.prop; })
        .map(function (change) { return ({
        sign: '-',
        path: change.path,
        value: formatJsValue(change.prop.oldValue)
    }); });
    var diffLines = __spreadArrays(added, removed);
    var longestPath = Math.max.apply(Math, diffLines.map(function (line) { return line.path.length; }));
    var diff = diffLines
        .sort(function (lineA, lineB) {
        return (lineA.path < lineB.path)
            ? -1
            : (lineA.path > lineB.path)
                ? 1
                : lineB.sign > lineA.sign
                    ? 1
                    : -1;
    })
        .map(function (_a) {
        var sign = _a.sign, path = _a.path, value = _a.value;
        return sign + (path + paddingWhitespace).substr(0, longestPath) + '   ' + value;
    })
        .join('\n');
    return diff;
}
var numericIndexRx = /^(?:0|[1-9]\d*)$/;
var safePropNameRx = /^\w[\w\d]*$/;
/**
 * Format an array as javascript property paths.
 * @example
 *     pathTo(['list', '3', 'id']) // => ".list[3].id"
 * @internal
 */
function pathToString(path) {
    return path.map(function (part) {
        if (numericIndexRx.test(part)) {
            return '[' + part + ']';
        }
        else if (safePropNameRx.test(part)) {
            return '.' + part;
        }
        return '[' + JSON.stringify(part) + ']';
    }).join('');
}
exports.pathToString = pathToString;
/**
 * Format any value to a single-line, understandable representation.
 * @internal
 */
function formatJsValue(value) {
    var type = typeof value;
    if (type === 'string') {
        return JSON.stringify(value);
    }
    else if (value === null) {
        return 'null';
    }
    else if (utils_1.isArray(value)) {
        return '[' + value.map(function (el) { return formatJsValue(el); }).join(',') + ']';
    }
    else if (type === 'function') {
        return 'function';
    }
    else if (value instanceof RegExp) {
        return value.toString();
    }
    else if (type === 'object') {
        return Object.prototype.toString.call(value);
    }
    return String(value);
}

//# sourceMappingURL=method-not-immutable-error.js.map
