"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createObservable = exports.observeImmutable = void 0;
var utils_1 = require("./utils");
function observeImmutable(instance, observableClass) {
    var instanceMetadata = utils_1.getInstanceMetadata(instance);
    if (!instanceMetadata) {
        var constructor = utils_1.objectGetPrototypeOf(instance).constructor;
        var className = constructor && utils_1.getFunctionName(constructor) || 'object class';
        throw new Error(className + ' is not decorated as @Immutable.');
    }
    return createObservable(instanceMetadata.observers, observableClass);
}
exports.observeImmutable = observeImmutable;
/**
 * Creates an observable of a passed Implementation or a plain-JavaScript observable.
 * @internal
 */
function createObservable(observerList, observableClass) {
    if (observableClass) {
        // Use a RXJS implementation passed by the user.
        return new observableClass(function (subscriber) { return handleSubscription(subscriber, observerList); });
    }
    // Use a plain JavaScript Observable.
    return {
        subscribe: function (subscriber) { return handleSubscription(subscriber, observerList); }
    };
}
exports.createObservable = createObservable;
/** @internal */
function handleSubscription(subscriber, observerList) {
    var callback = subscriber.next
        ? (function (value) { return subscriber.next(value); })
        : subscriber;
    observerList.push(callback);
    return {
        unsubscribe: function () {
            var index;
            while ((index = observerList.indexOf(callback)) >= 0) {
                observerList.splice(index, 1);
            }
        }
    };
}

//# sourceMappingURL=observe-immutable.js.map
