"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StateActionBranch = void 0;
var immutable_decorator_1 = require("./immutable-decorator");
var utils_1 = require("./utils");
/**
 * Abstract base class for state action branches of an {@link ImmutableStateStore}.
 */
var StateActionBranch = /** @class */ (function () {
    function StateActionBranch(_a) {
        var uses = _a.uses, initialState = _a.initialState;
        var prototype = utils_1.objectGetPrototypeOf(this);
        if (prototype === StateActionBranch.prototype) {
            throw new TypeError('StateActionBranch must be derived in a new class.');
        }
        if (!immutable_decorator_1.isConstructingImmutable()) {
            throw new TypeError(utils_1.getFunctionName(prototype.constructor) + ' inherits StateActionBranch but is not marked as Immutable.');
        }
        var usedKeys = utils_1.isArray(uses) ? uses : [uses];
        for (var _i = 0, usedKeys_1 = usedKeys; _i < usedKeys_1.length; _i++) {
            var key = usedKeys_1[_i];
            utils_1.objectDefineProperty(this, key, {
                configurable: true,
                enumerable: true,
                writable: true,
                value: initialState[key]
            });
        }
        utils_1.objectDefineProperty(this, 'initialState', {
            configurable: false,
            enumerable: false,
            writable: false,
            value: initialState
        });
    }
    return StateActionBranch;
}());
exports.StateActionBranch = StateActionBranch;

//# sourceMappingURL=state-action-branch.js.map
