import { deepEqual } from './deep-equal';
import { isArray, objectAssign, objectKeys } from './utils';
export function deepApplyWithReuse(a, b) {
    if (a === b) {
        return a;
    }
    else if (a == null) {
        return b;
    }
    else if (b == null) {
        return b;
    }
    else if (typeof a !== 'object') {
        return b;
    }
    else if (isArray(a)) {
        if (!isArray(b)) {
            return b;
        }
        let anyValueChanged = a.length !== b.length;
        const clone = new Array(b.length);
        for (let i = 0; i < clone.length; i++) {
            if (deepEqual(a[i], b[i])) {
                clone[i] = a[i];
            }
            else {
                anyValueChanged = true;
                clone[i] = b[i];
            }
        }
        return anyValueChanged ? clone : a;
    }
    else {
        const clone = objectAssign({}, a);
        const keysA = objectKeys(a);
        const keysB = objectKeys(b);
        let anyValueChanged = keysA.length !== keysB.length;
        for (const key of keysB) {
            const newValue = deepApplyWithReuse(a[key], b[key]);
            if (newValue !== a[key]) {
                anyValueChanged = true;
                clone[key] = newValue;
            }
        }
        return anyValueChanged ? clone : a;
    }
}

//# sourceMappingURL=deep-apply-with-reuse.js.map
