import { isArray, objectAssign, objectCreate, objectKeys, objectGetPrototypeOf } from './utils';
export function deepClone(original, depth = Number.POSITIVE_INFINITY) {
    if (!original || depth < 0) {
        return original;
    }
    else if (isArray(original)) {
        return original.map(val => deepClone(val, depth - 1));
    }
    const typeofOriginal = typeof original;
    if (typeofOriginal === 'object') {
        const prototype = objectGetPrototypeOf(original);
        if (prototype === Object.prototype) {
            return depth ? clonePropsTo({}, original, depth) : objectAssign({}, original);
        }
        else if (prototype === RegExp.prototype) {
            return new RegExp(original);
        }
        else if (prototype === Date.prototype) {
            return new Date(original);
        }
        else if (typeof Map === 'function' && prototype === Map.prototype) {
            return new Map(original);
        }
        else if (typeof Set === 'function' && prototype === Set.prototype) {
            return new Set(original);
        }
        else if (typeof Node === 'function' && original instanceof Node && typeof original.cloneNode === 'function') {
            return original.cloneNode(true);
        }
        const clone = objectCreate(prototype);
        return clonePropsTo(clone, original, depth);
    }
    else if (typeofOriginal === 'function') {
        const clone = function () {
            return original.apply(this, arguments);
        };
        return clonePropsTo(clone, original, depth);
    }
    else {
        return original;
    }
}
function clonePropsTo(target, source, depth) {
    for (let key of objectKeys(source)) {
        target[key] = deepClone(source[key], depth - 1);
    }
    return target;
}

//# sourceMappingURL=deep-clone.js.map
