import { isArray, objectKeys } from './utils';
/**
 * Returns true if the passed object contains any object reference more than once.
 */
export function hasSharedReferences(object) {
    if (typeof object !== 'object' || object == null) {
        return false;
    }
    return hasSharedRefs(object, []);
}
/** @internal */
function hasSharedRefs(object, traversed) {
    if (traversed.indexOf(object) >= 0) {
        return true;
    }
    traversed.push(object);
    if (isArray(object)) {
        for (let value of object) {
            if (typeof value === 'object' && value != null && hasSharedRefs(value, traversed)) {
                return true;
            }
        }
    }
    else {
        for (let key of objectKeys(object)) {
            if (typeof object[key] === 'object' && object[key] != null && hasSharedRefs(object[key], traversed)) {
                return true;
            }
        }
    }
    return false;
}

//# sourceMappingURL=has-shared-references.js.map
