import { flatEqual } from './flat-equal';
import { arraySlice, isArray, objectCreate, objectGetPrototypeOf, objectAssign, objectKeys } from './utils';
export function map(input, mapFn, thisArg) {
    let hasChanged = false;
    let clone;
    if (isArray(input)) {
        clone = arraySlice(input);
        let length = input.length;
        for (let index = 0; index < length; index++) {
            let element = input[index];
            let result = mapFn.call(thisArg, element, index, input);
            if (!flatEqual(result, element)) {
                hasChanged = true;
                clone[index] = result;
            }
            else {
                clone[index] = element;
            }
        }
    }
    else {
        clone = objectAssign(objectCreate(objectGetPrototypeOf(input)), input);
        let keys = objectKeys(input);
        for (let index = 0; index < keys.length; index++) {
            let prop = input[keys[index]];
            let result = mapFn.call(thisArg, prop, keys[index], input);
            if (!flatEqual(result, prop)) {
                hasChanged = true;
                clone[keys[index]] = result;
            }
            else {
                clone[keys[index]] = prop;
            }
        }
    }
    return hasChanged ? clone : input;
}

//# sourceMappingURL=map.js.map
