import { getFunctionName, objectGetPrototypeOf, getInstanceMetadata } from './utils';
export function observeImmutable(instance, observableClass) {
    const instanceMetadata = getInstanceMetadata(instance);
    if (!instanceMetadata) {
        const constructor = objectGetPrototypeOf(instance).constructor;
        const className = constructor && getFunctionName(constructor) || 'object class';
        throw new Error(className + ' is not decorated as @Immutable.');
    }
    return createObservable(instanceMetadata.observers, observableClass);
}
/**
 * Creates an observable of a passed Implementation or a plain-JavaScript observable.
 * @internal
 */
export function createObservable(observerList, observableClass) {
    if (observableClass) {
        // Use a RXJS implementation passed by the user.
        return new observableClass((subscriber) => handleSubscription(subscriber, observerList));
    }
    // Use a plain JavaScript Observable.
    return {
        subscribe: (subscriber) => handleSubscription(subscriber, observerList)
    };
}
/** @internal */
function handleSubscription(subscriber, observerList) {
    const callback = subscriber.next
        ? ((value) => subscriber.next(value))
        : subscriber;
    observerList.push(callback);
    return {
        unsubscribe() {
            let index;
            while ((index = observerList.indexOf(callback)) >= 0) {
                observerList.splice(index, 1);
            }
        }
    };
}

//# sourceMappingURL=observe-immutable.js.map
