import { isConstructingImmutable } from './immutable-decorator';
import { getFunctionName, isArray, objectDefineProperty, objectGetPrototypeOf } from './utils';
/**
 * Abstract base class for state action branches of an {@link ImmutableStateStore}.
 */
export class StateActionBranch {
    constructor({ uses, initialState }) {
        const prototype = objectGetPrototypeOf(this);
        if (prototype === StateActionBranch.prototype) {
            throw new TypeError('StateActionBranch must be derived in a new class.');
        }
        if (!isConstructingImmutable()) {
            throw new TypeError(getFunctionName(prototype.constructor) + ' inherits StateActionBranch but is not marked as Immutable.');
        }
        let usedKeys = isArray(uses) ? uses : [uses];
        for (let key of usedKeys) {
            objectDefineProperty(this, key, {
                configurable: true,
                enumerable: true,
                writable: true,
                value: initialState[key]
            });
        }
        objectDefineProperty(this, 'initialState', {
            configurable: false,
            enumerable: false,
            writable: false,
            value: initialState
        });
    }
}

//# sourceMappingURL=state-action-branch.js.map
